<?

require_once 'PEAR.php';

require_once 'O3Spaces/Utils.php';

class Config {

	private static $instance = NULL;

	private $initialized = false;
	private $authHandler = NULL;

	// Configuration, initialized with defaults
	private $config = array();

	private function __construct() {

		// Initialize defaults
		$this->cfg["store.tmp_dir"] = sys_get_temp_dir();
	}

	public static function getConfig() {
		if (Config::$instance == NULL) {
			Config::$instance = &new Config();
		}
		return Config::$instance;
	}

	public function get($key) {
		if (!$this->initialized) {
			PEAR::raiseError("Configuration not initialized");
		}
		return $this->config[$key];
	}

	public function initialize(&$keyedArray) {
		while (list($key, $value) = each($keyedArray)) {
			$this->set($key, $value);
		}

		$this->initializeAuthHandler();
		$this->initialized = true;
	}

	public function initializeAuthHandler() {
		$authStrategy = $this->get("auth.strategy");
		if ($authStrategy == "") {
			PEAR::raiseError("auth.strategy is not set in the configuration");
		}

		require_once "O3Spaces/Auth/$authStrategy.php"; // this is safe
		$strategy = new $authStrategy();

		if (!($strategy instanceof RequestAuthStrategy)) {
			PEAR::raiseError("Strategy is not an instanceof RequestAuthStrategy");
		}

		$this->authHandler = $strategy;

	}

	public function getAuthHandler() {
		return $this->authHandler;
	}

	public function set($key, $value) {
		$this->config[$key] = $value;
	}

	public function getWebDAVLocation() {
		return ($this->get("workplace.use_ssl") ? "webdavs://" : "webdav://" ) . $this->get("workplace.location") . "/webdav";
	}

	public function getHTTPLocation() {
		return ($this->get("workplace.use_ssl") ? "https://" : "http://" ) . $this->get("workplace.location");
	}

}

?>
