<?php
/**
 * O3Spaces Plugin
 *
 */

require_once "PEAR.php";
require_once "O3Spaces/Filter.php";

class Plugino3spaces extends Plugin {

	/**
	 * The Workplace MAPI properties
	 */
	var $properties = array(
		"WPPublishDateTime"  => "PT_SYSTIME:{00020329-0000-0000-C000-000000000046}:WPPublishDateTime",
		"WPPublishWorkspace" => "PT_STRING8:{00020329-0000-0000-C000-000000000046}:WPPublishWorkspace"
//		"WPPublishDateTime"  => "PT_SYSTIME:{00020329-0000-0000-C000-000000000046}:0x8358",
//		"WPPublishWorkspace" => "PT_STRING8:{00020329-0000-0000-C000-000000000046}:0x8359"
//		"WPPublishDateTime"  => "PT_SYSTIME:{00020329-0000-0000-C000-000000000046}:0x83580040",
//		"WPPublishWorkspace" => "PT_STRING8:{00020329-0000-0000-C000-000000000046}:0x8359001E"
//		"WPPublishDateTime"  => "PT_SYSTIME:{00020329-0000-0000-C000-000000000046}:0x83580040",
//		"WPPublishWorkspace" => "PT_STRING8:{00020329-0000-0000-C000-000000000046}:0x8359001E"

	);

	/**
	 * WorkplaceIntegration object and Mail plugin
	 */
	var $workplace;
	var $mail;
	var $faultyReason = "";

	/**
	* Constructor
	*/
	function Plugino3spaces(){
		// What filetype images do we have
		$this->icons = array("acp","bmp","docx","gif","jpg","mp2","mpeg","odf","ods","png","psd","swf","txt","wpd","xml","xslx","asf","csv","exe","htm","js","mp3","mpg"," odg","odt","ppt","rtf","tiff","url","xls","xsd","zip","avi","doc","ftl","html","lnk","mp4","msg","odp","pdf","pptx","shtml","tif","wmv","xlsx","xsl","vsd","eml");
	}


	/**
	* Function initializes the Plugin and registers all hooks
	*
	* @return void
	*/
	function init(){
		// on load the login data is fetched so they can be used to login
		$this->registerHook('server.index.login.success');
		$this->registerHook('server.main.include.jsfiles');
		$this->registerHook('server.dialog.general.setup.getmenubuttons');
		$this->registerHook('server.dialog.general.setup.getincludes');
		$this->registerHook('server.dialog.attachments.setup.getbody.uploadformhtml');
		$this->registerHook('server.dialog.general.upload_attachment.before');

		// trigger the entire execute event for maillistmodule
		$this->registerHook('server.module.maillistmodule.execute.before');

		// trigger the entire execute event for WorkspaceModule (uses login information from plugin sessiondata)    
		$this->registerHook('server.plugin.workspacemodule.execute');

	}


	/**
	* Function is executed when a hook is triggered by the PluginManager
	*
	* @param string $eventID the id of the triggered hook
	* @param mixed $data object(s) related to the hook
	* @return void
	*/
	function execute($eventID, &$data){
		switch($eventID){
			case "server.index.login.success":
				$this->login();
				break;
			case "server.main.include.jsfiles":
				$this->loadjs($data);
				break;
			case "server.dialog.general.setup.getmenubuttons":
				if($data['task'] == 'attachments'){
					$this->attachmentAddButton($data);
				}
				else{
					//$this->modifyDialogMenu($data);
				}
				break;
			case "server.dialog.general.setup.getincludes":
				$this->dialogGetIncludes($data);
				break;
			case "server.dialog.attachments.setup.getbody.uploadformhtml":
				$this->attachmentAddHiddenFields($data);
				break;
			case "server.dialog.general.upload_attachment.before":
				if ($this->addAttachmentFromWorkplace($data)) {
					// Register new recent location
					$this->setData("workplace_recent_insert_location",  $_REQUEST['workspace_location']);
				}
				break;
			case "server.module.maillistmodule.execute.before":
				$this->moduleExecute($data['moduleObject']);
				break;
			case "server.plugin.workspacemodule.execute":
				$this->moduleExecute($data['moduleObject']);
				break;
			default:
				if (DEBUG_PLUGINS) dump( __FILE__ . ":execute Received unknown event: $eventID. Data: " . print_r($data,true) );
				break;
		}
	}


	/**
	* Function is triggered (as well) when the login into the WA was sucessfully
	*
	* @return void
	*/
	function login() {
		// only login when executed with a username in the session!
		if (! $_SESSION["username"] ) return;

		$this->setData("authUser", $_SESSION["username"]);
		$this->setData("authPass", $_SESSION["password"]);

		return;
	}


	/**
	 * Initialize Workplace API
	 *
	 * @throws SoapFault when no ticket is available
	 * @return void
	 */
	function preOperation() {

		// What todo with session['username'] (after ctrl-f5 getData returns nothing anymore)
		if (! $this->getData("authUser")) { // not logged in ??
			if (DEBUG_PLUGINS) dump( __FILE__ . ":preOperation: Re-attempting login. This only happens after a CTRL-F5. Session username will be picked: " . $_SESSION["username"] );

			$this->login();
		}

		// Check if we should automatically update the plugin configuration with the 
		// Zarafa-loggedin user
		$zarafaUseUsername = ($GLOBALS["pluginconfig"]["o3spaces"]["zarafa.override.user"]);
		$zarafaUsePassword = ($GLOBALS["pluginconfig"]["o3spaces"]["zarafa.override.pass"]);

		// Set usernames and passwords. WorkspaceIntegration will figure out which one
		// to use based on trust.strategy setting.
		if ($zarafaUseUsername && $this->getData("authUser")) {
			$GLOBALS["pluginconfig"]["o3spaces"]["auth.trust.user"] = $this->getData("authUser");
			$GLOBALS["pluginconfig"]["o3spaces"]["auth.basic.user"] = $this->getData("authUser");
		}

		if ($zarafaUsePassword && $this->getData("authPass")) {
			$GLOBALS["pluginconfig"]["o3spaces"]["auth.basic.pass"] = $this->getData("authPass");
		}

		// Initialize Workplace connector
		$this->workplace = new WorkplaceIntegration($GLOBALS["pluginconfig"]["o3spaces"]);

		// Test authentication
		$authSuccessful = $this->workplace->testAuthentication();

		if (!$authSuccessful) {
			if (DEBUG_PLUGINS) dump( __FILE__ . ":preOperation: Authentication to Workplace failed. pluginconfig: ". print_r($GLOBALS["pluginconfig"]["o3spaces"],true) );

			$this->faultyReason = dgettext("plugin_o3spaces", "Failed to authenticate to Workplace");
			return false;
		}

		// Load Mail plugin
		$mail = $this->workplace->getPluginInstance("MailPlugin");

		if (PEAR::isError($mail) ) {
			if (DEBUG_PLUGINS) dump( __FILE__ . ":preOperation: MailPlugin cannot be loaded." );
			$this->faultyReason = dgettext("plugin_o3spaces", "Workplace Mailplugin missing on local server");
			return false;
		} else {
			$this->mail = $mail;
		}
		return true;
	}


	/** ----------------- GUI STUFF (BUTTONS ETC) --------------------------------------------------------------- */

	function loadjs(&$data) {
		$data["files"][] = $this->getPluginPath(). "workspacemodule.js";
	}

	function dialogGetIncludes(&$data) {
		// ignore all dialogs except "createmail"
		if ($data["task"] != "createmail" && $data["task"] != "attachments") return;

		$data["includes"][] = $this->getPluginPath(). "plugin.o3spaces.js";
		$data["includes"][] = $this->getPluginPath(). "plugin.o3spaces.css";
	}


	/**
	* Function adds "add download" fieldset button in the download dialog
	* 
	* @param mixed $data object(s) related to the hook
	* @return void
	*/
	function attachmentAddButton($data) {
		if($data["task"] != "attachments") return;
		$data["buttons"][] = array(
			'id' => "o3spaces_attachmentsdialog_addattachment",
			'name'  => dgettext('plugin_o3spaces', 'Insert from the Workplace'),
			'title' => dgettext('plugin_o3spaces', 'Add file(s) from the Workplace to this message'),
			'callback' => 'function(){webclient.openModalDialog(module, \'o3spacesattachment\', DIALOG_URL+\'task=workspace&plugin=o3spaces&type=insert\', 600,460, workspaceBrowseDialog_selectFileCallBack);}'
		);
	}

	/**
	* Function adds two hidden fields to publish a file to the workplace
	* 
	* @param mixed $data object(s) related to the hook
	* @return void
	*/
	function attachmentAddHiddenFields($data) {
		// add hidden fields
		$data['html'] .= '<input type="hidden" id="workspace_location" name="workspace_location" value="" />';
	}


	/**
	* Function triggers a download from the Workplace and adds it to the file list (download dialog)
	* 
	* @param mixed $data object(s) related to the hook
	* @return void
	*/
	function addAttachmentFromWorkplace(&$data) {

		if (!$this->preOperation())
			return false;

		// request is an workplace file
		if (!isset($_REQUEST['workspace_location'])) return;

		$fullpath = $_REQUEST['workspace_location'];
		if (!$fullpath) return;

		// Check if there are no files uploaded
		if(!isset($_SESSION["files"])) {
			$_SESSION["files"] = array();
		}
		
		// Check if no files are uploaded with this dialog_attachments
		if(!isset($_SESSION["files"][$_POST["dialog_attachments"]])) {
			$_SESSION["files"][$_POST["dialog_attachments"]] = array();
		}

		$filename = basename($fullpath);

		if (DEBUG_PLUGINS) dump( __FILE__ . ":addAttachmentFromWorkplace INFO Downloading [$filename] from [$fullpath]" );

		$info = $this->workplace->getFile($fullpath);

		if (PEAR::isError($info)) {
			if (DEBUG_PLUGINS) dump( __FILE__ . ":addAttachmentFromWorkplace ERROR Error in WorkplaceIntegration: " . $info->getMessage() );
			return;
		}

		$size = filesize($info["tmpfile"]);
		$contentType = $info["contenttype"];

		// Add file information to the session if download was successful
		if ($size != false) {
			$_SESSION["files"][ $_POST["dialog_attachments"] ][ basename($info["tmpfile"]) ] = array(
					"name" => utf8_to_windows1252(stripslashes(rawurldecode($filename))),
					"size" => $size,
					"type" => $contentType,
					"sourcetype" => 'o3spaces',
					"o3spacesFile" => true);
			return true;
		}


		return false;
	}

    /**
    * Function is triggered on 
    *     - MailListModule->execute and 
    *     - WorkspaceModule->execute 
    * works with the complete module objects
    *
    * @param object $aModule related module object
    * @return void
    */
    function moduleExecute(&$aModule) {

        foreach($aModule->data as $action)
        {
            if(isset($action["attributes"]) && isset($action["attributes"]["type"])) {
                $store = $aModule->getActionStore($action);
                $parententryid = $aModule->getActionParentEntryID($action);

				// locate store - after zarafa login store is null
				if (!$store) $store = $GLOBALS["mapisession"]->getDefaultMessageStore();


                switch($action["attributes"]["type"]) {
                    case "queryDirectoryTree":
                        $result = $this->queryDirectoryTree($aModule, $action);
                        break;

					case "WorkplacePublish":
						$result = $this->publishMessages($aModule, $store, $parententryid, $action);
						if ($result) {
							// Register new recent location
							$this->setData("workplace_recent_publish_location", $action["destinationFolder"]);
							if (DEBUG_PLUGINS) dump( __FILE__ . ":moduleExecute [WorkplacePublish] workplace_recent_publish_location = [" . $action["destinationFolder"] . "]" );
						}
                        break;

                    case "list":
						dump( __FILE__ . ":moduleExecute[action,list]: Retrieving MAPI-properties from store: $store: action=" . print_r($action,true));
						$props = $this->getCustomMAPIProps($store);
						$prop = $aModule->properties["o3spaces_workplace_publish_location"] = $props["WPPublishWorkspace"];
						$prop = $aModule->properties["o3spaces_workplace_publish_datetime"] = $props["WPPublishDateTime"];
						//$aModule->addColumn( $aModule->tablecolumns, "o3spaces_workplace_publish_location", true, 5, _("Workplace Published Location"), _("Sort On Workplace Published Location"), 80);
						//$aModule->addColumn( $aModule->tablecolumns, "o3spaces_workplace_publish_datetime", true, 5, _("Workplace Published Time"), _("Sort On Workplace Published Time"), 80);
						$GLOBALS["TableColumns"]->addColumn( $GLOBALS["TableColumns"], "o3spaces_workplace_publish_location", true, 5, _("Workplace Published Location"), _("Sort On Workplace Published Location"), 80);
						$GLOBALS["TableColumns"]->addColumn( $GLOBALS["TableColumns"], "o3spaces_workplace_publish_datetime", true, 5, _("Workplace Published Time"), _("Sort On Workplace Published Time"), 80);

						break;
                }
            }
        }
    }

	function getCustomMAPIProps($store) {

		/*
		// Get the ID's of the named properties for 'WPPublishDateTime' and 'WPPublishWorkspace' of mail items
		$guid = "{00020329-0000-0000-C000-000000000046}";
		$ids = mapi_getidsfromnames($store, array(0x8358, 0x8359), array($guid, $guid));

		// Create actual proptags by adding the type of the properties
		$res = array();
		$res["WPPublishDateTime"]  = mapi_prop_tag(PT_SYSTIME, mapi_prop_id($ids[0]));
		$res["WPPublishWorkspace"] = mapi_prop_tag(PT_STRING8, mapi_prop_id($ids[1]));

		return $res;
		*/

		$res = getPropIdsFromStrings($store, $this->properties);
		if (DEBUG_PLUGINS) dump( __FILE__ . ":getCustomMAPIProps: " . print_r($res,true));
		
		return $res;
	}

	/**
	* Retrieve a list of the workspace topology.
	* 
	* @param object $workspacemodule the workspacemodule object
	* @param array $action data transmitted from the request
	* @return boolean true on success or false on failure
	*/
	function queryDirectoryTree($workspaceModule, $action){

		if (DEBUG_PLUGINS) dump( __FILE__ . ":queryDirectoryTree INFO executing with action: " . str_replace(print_r($action,true),"\n","") );

		$data = array(
			"imguri" => $this->getPluginPath(),
			"spaces" => array(),
			"error" => 0,
			"errorMessage" => ""
		);

		if (! $this->preOperation()) {
			$data["error"] = 1;
			$data["errorMessage"] = $this->faultyReason;

			array_push($workspaceModule->responseData["action"], $data);
			$GLOBALS["bus"]->addData($workspaceModule->responseData);
			return false;
		}

		$viewFiles = true;
		$parentId = 0;
		$folder = "";
		$restoreState = false;
		$mode = "publish";

		if (isset($action["viewFiles"]))
			$viewFiles = $action['viewFiles'];

		if (isset($action["parentId"])) 
			$parentId = $action['parentId'];

		if (isset($action["folder"]))
			$folder = $action['folder'];

		if (isset($action["restoreState"]))
			$restoreState = $action['restoreState'] ? true : false;

		if (isset($action["mode"])) 
			$mode = $action["mode"];

		$key = "workplace_recent_" . $mode . "_location";
		$restoreDirectory = $this->getData($key);

		$filter = $viewFiles == 0 ? new FolderFilter() : null;

		if (DEBUG_PLUGINS) dump( __FILE__ . ":queryDirectoryTree INFO Retrieving listing for folder [$folder], restoring up to [$restoreDirectory]");

		$spaces = $this->queryWorkplace( $folder, $filter, $restoreDirectory, $parentId );

		if (DEBUG_PLUGINS) dump( __FILE__ . ":queryDirectoryTree INFO Retrieved listing for folder [$folder]: " . print_r($spaces,true) );

		$spacelist = array();
		foreach ($spaces as $space) {

			$space["icon"] = "../page";

			if (isset($space["type"])) {
				if ($space["type"] == "dir") {
					$space['icon'] = '../folder';
					$space['icon_open'] = '../folderopen';
				}

				// get extension
				$pi = pathinfo( $space["webdav_path"] );
				if (isset($pi["extension"]) && in_array($pi["extension"], $this->icons)) {
					$space["icon"] = $pi["extension"];
				}
			}

			$space['size'] = $space["props"]["contentlength"] ? "".$space["props"]["contentlength"] : 0;
			$space['webdav_path'] = $space['webdav_path'];

			$space['isdocument'] = $space["type"] == "file" ? true : false;

			$space['id'] = "".$space['props']['O3:id'];
			unset($space['props']);

			array_push($spacelist, $space);
		}

		$data["spaces"] = $spacelist;
		$data["attributes"] = array("type" => "listspaces");

		if (DEBUG_PLUGINS) dump( __FILE__ . ":queryDirectoryTree Data to bus: " . print_r($data,true) );

		array_push($workspaceModule->responseData["action"], $data);
		$GLOBALS["bus"]->addData($workspaceModule->responseData);

		return true;
	}


	/** Query workplace for directory listings, is used recursively, can write a parentId key to each item */
	function queryWorkplace( $path, $filter, $restorePath, $parentId ) {

		$contents = array();
		if ($path == "" || $path == "mail") {
			$contents = $this->mail->getMailRootFolders(true); // Query underlying WIS-properties as well, more expensive
		} else {
			$contents = $this->workplace->getFolderContents( $path, $filter );
		}

		// sort contents, first on directory, then on file name, and yes, case insensitive please
		usort($contents, 'sortDirectoryContents');

		$result = array();

		foreach ($contents as $entry) {

			$entry['parentId'] = $parentId;

			array_push($result, $entry);

			// If the current path is the left side of the $restorePath we have a hit, go into recursive mode then
			if ($restorePath && (strpos($restorePath, $entry['webdav_path']) === 0)) {

				if (DEBUG_PLUGINS) dump( __FILE__ . ":queryWorkplace. webdavPath [".$entry['webdav_path'] . " is found as the start of restorePath=[$restorePath]");

				// Get subdirectory information and merge the results
				$res = $this->queryWorkplace( $entry['webdav_path'], $filter, $restorePath, $entry['props']['O3:id'] );
				$result = array_merge($result, $res);
			}
		}

		dump(__FILE__. ":queryWorkplace. Incremental result: " . print_r($result,true));

		return $result;

	}

	/** Archive a bunch of messages in the Workplace
	 * 
	 * @param object $workspacemodule workspacemodule object
	 * @param string $store the store of the selected email
	 * @param string $parententryid the parent entryid of the selected email
	 * @param array $action data from the request 
	 * @return boolean true on success or false on failure
	 */
	function publishMessages(&$workspacemodule, $store, $parententryid, $action) {

		$response = array();
		$response["attributes"] = array("type" => "WorkplacePublicationResponse");
		$response["isError"] = 0;
		$response["errorMessage"] = "";

		// check the single equal-signs
		if( ($destFolderNotSet = !isset($action["destinationFolder"])) or 
			($entryIdsNotSet   = !isset($action["entryids"])) or
			($mapiFunctionNotExists = !function_exists("mapi_inetmapi_imtoinet"))
			) {

			if (DEBUG_PLUGINS &&  $mapiFunctionNotExists) dump( __FILE__ . ":archiveMessages PHP-MAPI not compliant: mapi_inetmapi_imtoinet does not exist (needed for message/rfc822 export)" );
			if (DEBUG_PLUGINS && !$mapiFunctionNotExists) dump( __FILE__ . ":archiveMessages ERROR null values in destinationFolder={".$action["destinationFolder"]."} entryids=".$action["entryids"] );

			$response["isError"] = 1;
			if ($destFolderNotSet)      $response["errorMessage"] .= dgettext("plugin_o3spaces", "No destination folder set");
			if ($entryIdsNotSet)        $response["errorMessage"] .= dgettext("plugin_o3spaces", "No messsages selected");
			if ($mapiFunctionNotExists) $response["errorMessage"] .= dgettext("plugin_o3spaces", "PHP-MAPI function missing");

			array_push($workspacemodule->responseData["action"], $response);
			$GLOBALS["bus"]->addData($workspacemodule->responseData);
			return false;
		}

		// init env and check if there is a reasonable chance that we will succeed
		if (!$this->preOperation())
			return false;

		$destFolder = $action["destinationFolder"];
		$msgIds = array();

		if (is_array($action["entryids"])) $msgIds = $action["entryids"];
		else if ($action["entryids"])      $msgIds = array($action["entryids"]);

		//
		// Get each message from the MAPI message store, convert it to message/rfc822 and trigger
		// Workplace to upload.
		//
		$msgCount = 0;
		foreach($msgIds as $entryId) {

			// Write message in tmpfile
			$tmpFile = tempnam(TMP_PATH, "o3spaces-mapi-");
			$fp = fopen($tmpFile,"w");
			$return  = $this->writeMessageAsRFC822( $store, $entryId, $fp, array(PR_SUBJECT) );
			fclose($fp);

			// Create filename for Workplace based on subject
			$filename = $this->createFilename( trim(windows1252_to_utf8($return[PR_SUBJECT],"\0")), $destFolder );

			// Upload file in workplace
			if ($this->uploadFile($filename, $tmpFile, $destFolder)) {
				$msgCount++;

				// Mark the message with the proper labels
				list($space) = split("/", rawurldecode($destFolder));
				if (!$this->modifyMessageAddWorkspaceProps( $store, $entryId, $space, mktime() )) {
					if (DEBUG_PLUGINS) dump( __FILE__ . ":archiveMessages ERROR modifyMessageAddLocation failed");
				}
			}

			unlink($tmpFile);
		}

		//
		// Make response object and push it onto the bus
		//
		$response["archivedMessages"] = $msgCount;

		array_push($workspacemodule->responseData["action"], $response);
		$GLOBALS["bus"]->addData($workspacemodule->responseData);

		return true;

	}

	/** Create a filename for a given subject.
	 * 
	 * Will return "$subject.eml" with a bunch of crappy characters removed.
	 * 
	 * @param $subject - Subject that will become the filename.
	 * @param $destFolder - Requested destination folder in Workplace. Currently not used. TODO
	 */
	function createFilename( $subject, $destFolder ) {
		
		if (!$subject) $subject = "(no subject)";

		$filename = trim($subject); // TODO: Check if not already exists
		$filename = str_replace( array('*',':','[',']','&','^','%','$','#','@','!'), "", $filename); 
		$filename .= ".eml";

		if (DEBUG_PLUGINS) dump( __FILE__ . ":createFilename INFO Created filename [$filename] for subject [$subject]" );

		return $filename;
	}

	/** Write a MAPI message from a store to an open filepointer - return requested MAPI-propertyvalues
	 *
	 * @param $store   - MAPI store object to get message from
	 * @param $entryId - The MAPI message-id to export
	 * @param $fp      - Open filepointer to write data message/rfc822 data to
	 * @param $props   - Array containing the properties to return, e.g. PR_SUBJECT (no conversion happening here)
	 */
	function writeMessageAsRFC822( $store, $entryId, $fp, $props = array() ) {

		if (DEBUG_PLUGINS) dump(__FILE__ .":writeMessageAsRFC822 INFO Loading MAPI entryId: $entryId");
		
		// get message
		$message      = mapi_msgstore_openentry($store, hex2bin($entryId) );
		$messageprops = mapi_getprops($message, $props);

		// get message, addressbook and open message/rfc822 stream
		$addrBook = mapi_openaddressbook($GLOBALS["mapisession"]->getSession());
		$stream   = mapi_inetmapi_imtoinet($GLOBALS["mapisession"]->getSession(), $addrBook, $message, array());

		// store message in temporary file
		$stat = mapi_stream_stat($stream);
		$size = $stat["cb"];

		for($i = 0; $i < $size; $i += BLOCK_SIZE) {
			$content = mapi_stream_read($stream, BLOCK_SIZE);
			fputs($fp, $content);
		}

		return $messageprops;
	}

	/** Add the Workplace-location information to a message 
	 *
	 */
	function modifyMessageAddWorkspaceProps( $store, $entryId, $location, $time ) {

		if (DEBUG_PLUGINS) dump(__FILE__ .":modifyMessageAddLocation INFO Loading MAPI entryId: $entryId");

		// get message
		$message = mapi_msgstore_openentry($store, hex2bin($entryId) );

		$props = $this->getCustomMAPIProps($store);

		$change = array(
			$props["WPPublishWorkspace"] => $location,
			$props["WPPublishDateTime"]  => $time,
		);
		if (mapi_setprops($message, $change)) {
			dump( __FILE__ . ":modifyMessageAddLocation INFO Pre-saving - Added MAPI properties: " . print_r($change,true));
		}

		return mapi_savechanges($message);
	}

	/**
	 * Upload a file to the Workplace
	 *
	 * @param string $fileName  The filename of the file to be created
	 * @param string $tmpFile   The local path to the file to upload
	 * @param string $destLocation The Workspace directory to upload the file to
	 * @return boolean
	 */
	function uploadFile( $fileName, $tmpFile, $destLocation ){

		if (DEBUG_PLUGINS) dump(__FILE__ . ":uploadFile INFO Uploading file $tmpFile as $fileName to $destLocation");

		// Upload the file [$name, $sourcePath, $destLocation, $mode=WORKPLACE_PUT_MODE_NEWREVISION)]
		$r = $this->workplace->putFile( $fileName, $tmpFile, $destLocation ); // TODO: Give mode choice please
		if (PEAR::isError($r)) {
			if (DEBUG_PLUGINS) dump( __FILE__ . ":uploadFile ERROR Upload failed. Message: " . $r->getMessage() );
			return false;
		}

		return true;
	}

}

// ************************************************************

/** Sort the output of the workplace filebrowser
 *
 * sort on type (dir before file), then on name (case-insensitive, a before z) 
 */
function sortDirectoryContents($a, $b) {

	if ($a["type"] == $b["type"]) {

		$aName = strtolower($a["name"]);
		$bName = strtolower($b["name"]);

		# both same sort, sort on file
		if ($aName == $bName) {
			return 0;
		} elseif ($aName < $bName) {
			return -1;
		} else {
			return 1;
		}
	} elseif ($a["type"] < $b["type"]) {
		return -1;
	} elseif ($a["type"] > $b["type"]) {
		return 1;
	}

	return 0;

}
?>
